# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe.v2.payments._off_session_payment import (
    OffSessionPayment as OffSessionPayment,
)
from stripe.v2.payments._off_session_payment_service import (
    OffSessionPaymentService as OffSessionPaymentService,
)
